import pytest

from slash import get_application_context

from slash.reporting.console_reporter import ConsoleReporter
from slash.reporting.null_reporter import NullReporter


@pytest.mark.parametrize('use_reporter', [True, False])
def test_app_context_reporter(use_reporter):
    with get_application_context(report=use_reporter, argv=[]) as app:
        assert isinstance(app.session.reporter, ConsoleReporter if use_reporter else NullReporter)



from numbers import Number
import string

from .._compat import string_types
from .fixtures.parameters import Parametrization

_PRINTABLE_TYPES = (Number,) + string_types
_PRINTABLE_CHARS = set(string.ascii_letters) | set(string.digits) | set("-_")


class Variation(object):

    """Represents a single variation of parameter points. A variation is merely a mapping of fixture ids to their values.
    This mostly applies for parametrization fixtures. The other fixtures follow since they are either constant
    or indirectly depend on parametrization"""

    def __init__(self, store, param_value_indices, name_bindings=None, verbose_id=None):
        """
        :param name_bindings: dictionary mapping parameter name to its id
        """
        super(Variation, self).__init__()
        self._store = store
        self.param_value_indices = param_value_indices
        self.name_bindings = name_bindings or {}
        self.values = {}
        self.id = self._store.get_variation_id(self)
        self.safe_repr = self._get_safe_repr()
        self.verbose_id = verbose_id

    def _get_safe_repr(self):
        returned = {}
        for name, f in self.name_bindings.items():
            if isinstance(f, Parametrization):
                returned[name] = self._format_parameter_safe(name, f)
            else:
                returned[name] = '{}{}'.format(name, self.id[name])
        return ','.join('{}={}'.format(key, returned[key]) for key in sorted(returned))

    def _format_parameter_safe(self, name, p):
        param_index = self.param_value_indices[p.info.id]
        value = str(self._store.get_value(self, p))
        if self._is_printable(value):
            return str(value)
        return '{}{}'.format(name, param_index)

    def _is_printable(self, value):
        if not isinstance(value, _PRINTABLE_TYPES):
            return False
        return _PRINTABLE_CHARS.issuperset(value)

    def has_value_for_parameter(self, param_id):
        return param_id in self.param_value_indices

    def populate_early_known_values(self):
        for name, param in self.name_bindings.items():
            if isinstance(param, Parametrization):
                self.values[name] = self._store.get_value(self, param)

    def populate_values(self):
        for name, param in self.name_bindings.items():
            if name not in self.values:
                self.values[name] = self._store.get_value(self, param)

    def forget_values(self):
        self.values.clear()

    def get_param_value(self, param):
        return self._store.get_value(self, param)

    def __eq__(self, other):
        if isinstance(other, Variation):
            other = other.param_value_indices
        if not isinstance(other, dict):
            return NotImplemented
        return self.param_value_indices == other

    def __ne__(self, other):
        return not (self == other)  # pylint: disable=superfluous-parens,unneeded-not

    def __repr__(self):
        return 'Variation({0})'.format(', '.join('{0}={1}'.format(key, value) for key, value in self.param_value_indices.items()))

    def __nonzero__(self):
        return bool(self.param_value_indices)

    __bool__ = __nonzero__



import warnings
from slash.utils.warning_capture import warning_callback_context

def test_warning_capture_context():
    captured = []

    def callback(*args, **kwargs):
        captured.append(Warning(*args, **kwargs))

    with warning_callback_context(callback):
        warnings.warn('some warning')

    assert len(captured) == 1
    [w] = captured # pylint: disable=unbalanced-tuple-unpacking
    assert w.args[0].args[0] == 'some warning'

class Warning(object):

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs



#pylint: disable=unused-variable, unused-argument
import slash

from .utils import resolve_and_run

def test_fixture_exclusion():

    with slash.Session() as s:

        @s.fixture_store.add_fixture
        @slash.parametrize('param', [1, 2])
        @slash.fixture
        def fixture(param):
            return param * 10

        @slash.exclude('fixture.param', [2])
        def test_something(fixture):
            slash.context.result.data['value'] = fixture

        results = resolve_and_run(test_something)
    assert len(results) == 2
    assert results[0].is_success()
    assert results[0].data['value'] == 10
    assert results[1].is_skip()
    assert not results[1].is_started()



from slash.frontend.slash_list_plugins import slash_list_plugins
from slash.plugins import manager
from slash._compat import StringIO


def test_slash_list_plugins():
    report_stream = StringIO()

    slash_list_plugins([], report_stream=report_stream)
    output = report_stream.getvalue()
    assert output
    installed = manager.get_installed_plugins()
    for plugin_name in installed:
        assert plugin_name in output



import sys
import pytest
import coverage

import slash.plugins

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'), reason='Cannot run on pypy')
def test_coverage_plugin(suite, enabled_coverage_plugin): # pylint: disable=unused-argument
    suite.run()

@pytest.fixture
def enabled_coverage_plugin(request, patched_coverage, config_override):
    slash.plugins.manager.activate('coverage')
    config_override('plugin_config.coverage.report', False)

    @request.addfinalizer
    def deactivate(): # pylint: disable=unused-variable
        slash.plugins.manager.deactivate('coverage')

@pytest.yield_fixture
def patched_coverage(forge):
    s = forge.replace_with(coverage.Coverage, 'start', lambda self: None)
    forge.replay()
    try:
        yield
    finally:
        forge.restore_all_replacements()



from uuid import uuid4
import pytest


@pytest.mark.parametrize('failure_type', ['failure', 'error'])
@pytest.mark.parametrize('use_custom_message', [True, False])
def test_add_error_custom_exc_info(suite, suite_test, failure_type, use_custom_message):
    message = str(uuid4())

    if use_custom_message:
        add_error_args = '{!r}, '.format(message)
    else:
        add_error_args = ''

    add_error_args += 'exc_info=exc_info'

    code = """
import sys
try:
    1/0
except ZeroDivisionError:
    exc_info = sys.exc_info()
    try:
        None.a = 2
    except AttributeError:
        slash.add_{0}({1})
    """.format(failure_type, add_error_args)

    for line in code.strip().splitlines():
        suite_test.append_line(line)

    if failure_type == 'error':
        suite_test.expect_error()
    else:
        suite_test.expect_failure()
    summary = suite.run()
    [result] = summary.get_all_results_for_test(suite_test)

    if failure_type == 'error':
        [err] = result.get_errors()
    else:
        [err] = result.get_failures()

    assert err.has_custom_message() == use_custom_message
    if use_custom_message:
        assert err.message == message
    assert err.exception_type is ZeroDivisionError
    assert err.traceback.frames




from setuptools import setup, find_packages
import functools
import os
import sys

_PYTHON_VERSION = sys.version_info
_IS_PYPY = hasattr(sys, 'pypy_version_info')
_in_same_dir = functools.partial(os.path.join, os.path.dirname(__file__))

with open(_in_same_dir("slash", "__version__.py")) as version_file:
    exec(version_file.read())  # pylint: disable=W0122

install_requires = [
    "arrow",
    "colorama",
    "confetti>=2.4.1",
    "dessert>=1.0.2",
    "emport>=1.1.3",
    "gossip>=2.0.0",
    "Logbook>=0.11.2",
    "orderedset>=2.0.0",
    "requests>=1.1.0",
    "raven",
    "py>=1.4.20",
    "pyparsing",
    # DO NOT ADD pyforge, lxml or any other package only required for testing
]

if _PYTHON_VERSION < (2, 7):
    install_requires.append("argparse")
    install_requires.append("ordereddict")

# Ipython dep
if _PYTHON_VERSION < (2, 7) or _IS_PYPY:
    install_requires.insert(0, "ptyprocess==0.4") # 0.5 dropped 2.6 support
    install_requires.append('IPython==1.2.1')
else:
    install_requires.append('IPython')

if _PYTHON_VERSION < (3, 3):
    install_requires.append("contextlib2")

setup(name="slash",
      classifiers=[
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3.3",
          "Programming Language :: Python :: 3.4",
          "Programming Language :: Python :: 3.5",
      ],
      description="A Testing Framework",
      license="BSD",
      author="Rotem Yaari",
      author_email="vmalloc@gmail.com",
      url="http://getslash.github.io/",
      version=__version__,  # pylint: disable=E0602
      packages=find_packages(exclude=["tests"]),
      install_requires=install_requires,
      entry_points=dict(
          console_scripts=[
              "slash  = slash.frontend.main:main_entry_point",
          ]
      ),
)



import os

from . import pattern_matching

def iter_suite_file_paths(suite_files):
    for filename in suite_files:

        dirname = os.path.abspath(os.path.dirname(filename))
        for path in open(filename):
            path = path.strip()
            if not path or path.startswith("#"):
                continue

            path, filter = _parse_path_and_filter(path)

            if not os.path.isabs(path):
                path = os.path.abspath(os.path.join(dirname, path))

            if not path.endswith('.py') and '.py:' not in path and not os.path.isdir(path):
                for p, other_filter in iter_suite_file_paths([path]):
                    yield p, _and_matchers(filter, other_filter)
                continue

            yield path, filter


def _and_matchers(a, b):
    if a is None:
        return b
    if b is None:
        return a
    return pattern_matching.AndMatching([a, b])


def _parse_path_and_filter(line):
    if '#' not in line:
        return line, None

    line, remainder = line.split('#', 1)
    line = line.strip()
    remainder = remainder.strip()
    if not remainder.startswith('filter:'):
        return line, None
    return line, pattern_matching.Matcher(remainder.split(':', 1)[1])



import os
import re
from slash.frontend.slash_list import slash_list
from slash._compat import StringIO
from .utils.suite_writer import Suite

import pytest


@pytest.mark.parametrize('flag', ["--only-fixtures", "--only-tests", None])
def test_slash_list(suite, flag):
    suite.debug_info = False
    _ = suite.slashconf.add_fixture()
    path = suite.commit()
    report_stream = StringIO()

    args = [path]
    if flag is not None:
        args.append(flag)
    slash_list(args, report_stream)
    assert report_stream.getvalue()


@pytest.mark.parametrize('allow_empty', [True, False])
def test_slash_list_without_any_tests(allow_empty):
    empty_suite = Suite()
    empty_suite.debug_info = False
    path = empty_suite.commit()
    report_stream = StringIO()
    args = [path]
    if allow_empty:
        args.append('--allow-empty')
    rc = slash_list(args, report_stream)
    expected_rc = 0 if allow_empty else 1
    assert rc == expected_rc


@pytest.mark.parametrize('should_show_tags', [True, False])
def test_slash_list_tests(suite, should_show_tags, suite_test):
    suite_test.add_decorator('slash.tag("bla")')
    suite.debug_info = False
    path = suite.commit()
    report_stream = StringIO()
    args = [path, '--only-tests']
    if should_show_tags:
        args.append('--show-tags')
    slash_list(args, report_stream)
    output = report_stream.getvalue()
    assert ('Tags' in output) == (should_show_tags)


@pytest.mark.parametrize('relative', [True, False])
def test_slash_list_tests(suite, suite_test, relative):
    suite.debug_info = False
    path = suite.commit()
    report_stream = StringIO()
    args = [path]
    if relative:
        args.append('--relative-paths')
    slash_list(args, report_stream)
    output_lines = {
        _strip(line)
        for line in report_stream.getvalue().splitlines()
    } - {"Tests", "Fixtures"}
    assert output_lines
    for filename in output_lines:
        assert os.path.isabs(filename) == (not relative)

def _strip(line):
    return re.sub(r'\x1b\[.+?m', '', line).strip()
